@@ -12,7 +12,8 @@ def import_from_xls(reqmodule, req_xls):
     req = {}
     for row in range(1, sheet.nrows):
         for col, field in enumerate(fields):
-            req[field] = sheet.cell_value(row, col)
+            if field != 'non_stored_fields':
+                req[field] = sheet.cell_value(row, col)
         reqmodule.add_req(req)
 
 
@@ -46,5 +47,5 @@ def add_test_results(module_path, test_results_file):
     else:
         test_result_files = [test_results_file]
 
-    with open(module_path + '/test_results.temp.json', 'w') as testlist_file:
+    with open(module_path + '/test_results.temp.yaml', 'w') as testlist_file:
         yaml.dump(test_result_files, testlist_file)
